package com.dayou.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.dayou.common.BaseEntity;
import com.dayou.entity.FinanceClaim;
import com.dayou.entity.FinanceRealFund;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.FinanceRealFundMapper;
import com.dayou.service.IFinanceClaimService;
import com.dayou.service.IFinanceRealFundService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.LoginContext;
import com.dayou.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 财务实收款 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-12-08
 */
@Service
public class FinanceRealFundServiceImpl extends ServiceImpl<FinanceRealFundMapper, FinanceRealFund> implements IFinanceRealFundService {

    @Autowired
    private FinanceRealFundMapper financeRealFundMapper;

    @Autowired
    private IFinanceClaimService financeClaimService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<FinanceRealFundVO> selectPage(Page page, FinanceRealFund financeRealFund,String keyword){
        Page<FinanceRealFundVO> ret = financeRealFundMapper.getPage(page,financeRealFund,keyword);
        return ret;
    }


    @Override
    public FinanceRealFund detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(FinanceRealFund financeRealFund){
        financeRealFund.setCreatorId(LoginContext.getCurrentUserId());
        return  this.save(financeRealFund);
    }

    @Override
    public Boolean update(FinanceRealFund financeRealFund){
        List<FinanceClaim> list = financeClaimService.list(new LambdaQueryWrapper<FinanceClaim>().eq(FinanceClaim::getRealFundId, financeRealFund.getId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (CollectionUtil.isNotEmpty(list)){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"此笔实收款，已有订单认领金额，无法修改。");
            return Boolean.FALSE;
        }
        return  this.updateById(financeRealFund);
    }

    @Override
    public Boolean delete(Long id){
        List<FinanceClaim> list = financeClaimService.list(new LambdaQueryWrapper<FinanceClaim>().eq(FinanceClaim::getRealFundId, id).eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (CollectionUtil.isNotEmpty(list)){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"此笔实收款，已有订单认领金额，无法删除。");
            return Boolean.FALSE;
        }
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public BigDecimal getNotClaimAmount(Long realFundId) {
        BigDecimal claimAmount = financeRealFundMapper.getNotClaimAmount(realFundId);
        return claimAmount;
    }

    @Override
    public Page<MajorProdOverdueVO> overdueMajor(MajorProdOverdueVO overdueVO, Page page) {
        Page<MajorProdOverdueVO> result = financeRealFundMapper.overdueMajor(page,overdueVO);
        return result;
    }

    @Override
    public List<MajorProdOverdueVO> overdueMajorExport(MajorProdOverdueVO overdueVO) {
        return financeRealFundMapper.overdueMajorExport(overdueVO);
    }

    @Override
    public Page<SettleMajorProductionVO> majorEvaluatorSettleProd(SettleMajorProductionVO settleVO, Page page) {
        Page<SettleMajorProductionVO> result = financeRealFundMapper.majorEvaluatorSettleProd(page,settleVO);
        return result;
    }

    @Override
    public Page<SettleMajorProductionVO> majorMarketSettleProd(SettleMajorProductionVO settleVO, Page page) {
        Page<SettleMajorProductionVO> result = financeRealFundMapper.majorMarketerSettleProd(page,settleVO);
        return result;
    }

    @Override
    public List<SettleMajorProductionVO> majorMarketSettleExport(SettleMajorProductionVO production) {
        return financeRealFundMapper.majorMarketSettleExport(production);
    }

    @Override
    public List<SettleMajorProductionVO> majorEvaluatorSettleExport(SettleMajorProductionVO production) {
        return financeRealFundMapper.majorEvaluatorSettleExport(production);
    }

    @Override
    public List<UserCommissionTotalAmountVO> majorEvaluatorCommission(UserCommissionTotalAmountVO vo) {
        return financeRealFundMapper.majorEvaluatorCommission(vo);
    }

    @Override
    public List<UserCommissionProdVO> majorEvaluatorCommissionDetail(UserCommissionProdVO vo) {
        return financeRealFundMapper.majorEvaluatorCommissionDetail(vo);
    }

    @Override
    public List<UserCommissionTotalAmountVO> majorEvaluatorTotalAmountExport(UserCommissionTotalAmountVO vo) {
        return financeRealFundMapper.majorEvaluatorTotalAmountExport(vo);
    }

    @Override
    public List<UserCommissionProdVO> evaluatorCurUserDetailExport(UserCommissionProdVO vo) {
        return financeRealFundMapper.evaluatorUserDetailExport(vo);
    }

    @Override
    public List<UserCommissionProdVO> evaluatorAllUserDetailExport(UserCommissionProdVO vo) {
        vo.setUserId(null);
        return financeRealFundMapper.evaluatorUserDetailExport(vo);
    }

    @Override
    public List<UserCommissionTotalAmountVO> majorMarketCommission(UserCommissionTotalAmountVO vo) {
        return financeRealFundMapper.majorMarketCommission(vo);
    }

    @Override
    public List<UserCommissionProdVO> majorMarketCommissionDetail(UserCommissionProdVO vo) {
        return financeRealFundMapper.majorMarketCommissionDetail(vo);
    }

    @Override
    public List<UserCommissionTotalAmountVO> majorMarketTotalAmountExport(UserCommissionTotalAmountVO vo) {
        return financeRealFundMapper.majorMarketTotalAmountExport(vo);
    }

    @Override
    public List<UserCommissionProdVO> marketCurUserDetailExport(UserCommissionProdVO vo) {
        return financeRealFundMapper.marketUserDetailExport(vo);
    }

    @Override
    public List<UserCommissionProdVO> marketAllUserDetailExport(UserCommissionProdVO vo) {
        vo.setUserId(null);
        return financeRealFundMapper.marketUserDetailExport(vo);
    }
}
